class Tower { 
  int loc; // location, i.e. x value
  int lvl; // level, i.e. height
  int typ; // type, either 0,1, or 2
  Tower (int tp) {  
    loc = 0; 
    lvl = 0;
    typ = tp; 
  } 
  void moveTo(int lc) {
    loc = lc;
  }
  void setLevel(int ll) {
    lvl = ll;
  }
  void show() { 
    tint(255,128);
    image(getTower(typ),loc,height-lvl,50,lvl);
  } 
  void showEmphasised(){
    tint(255,255);
    image(getTower(typ),loc,height-lvl,50,lvl);  
  }
}

PImage city;
PImage truck_one; 
PImage truck_two;
PImage tower_zero;
PImage tower_one;
PImage tower_two;

int maxtowers=10;
Tower[] twr; 
int towers=0;

int x=0;
int curtyp=0;
boolean rightward = true;

PImage getTower(int tp) {
  PImage b;
  switch(tp) {
  case 0 : 
    b = tower_zero; 
    break;
  case 1 : 
    b = tower_one;  
    break;
  case 2 : 
    b = tower_two;  
    break;
  default: 
    b = tower_zero;
  }
  return b;
}

void setup() {
  size(750,200);
  twr = new Tower[maxtowers]; 
  city       = loadImage("city.jpg");
  truck_one  = loadImage("truck_one.jpg");
  truck_two  = loadImage("truck_two.jpg");
  tower_zero = loadImage("tower_zero.jpg");
  tower_one  = loadImage("tower_one.jpg");
  tower_two  = loadImage("tower_two.jpg");
}

void draw() {
  // display background
  tint(255,128);
  image(city,0,0);

  // move truck autonomously
  if (random(0,5) > 4) {
    if (rightward)
      x=x+2;
    else x=x-2;
  }
  if (x>width) x=width;
  if (x<0) x=0;
  tint(255,128);

  // display truck
  if (rightward)  
    image(truck_two,x,height-25,50,25);
  else image(truck_one,x,height-25,50,25);

  // display preview
  tint(255,128);
  image(getTower(curtyp),x,50,25,50);
  // display towers for indexes 0..towers-1

  for (int i=0; i<towers; i++) {
    twr[i].show();
  }
}

// several sorting algorithms to be used in keypressed for keyCode ENTER
void selectionSort(){
  // sort loc[0..towers-1], lvl[idem], typ[idem] by increasing lvl value
  for (int i = 0; i < towers; i++) {
    // when here: i smallest values are sorted in 0..i-1
    int mi = i;
    for (int j = i; j < towers; j++) {
      // when here: mi is index of smallest in i..j-1
      if (twr[j].lvl < twr[mi].lvl)  
        mi = j;
    }
    // swap towers at indexes i and mi
    Tower tmp = twr[i]; 
    twr[i] = twr[mi]; 
    twr[mi] = tmp;
  }
}

void keyPressed() {
  if(keyCode == UP)    { 
    curtyp=(curtyp+1)%3; 
  } 
  if(keyCode == DOWN)  { 
    curtyp=((curtyp+3)-1)%3; 
  } 
  if(keyCode == LEFT)  { 
    rightward=false; 
  } 
  if(keyCode == RIGHT) { 
    rightward=true; 
  } 
  if(key == ' ')       { // add tower  
    if (towers<maxtowers){
      Tower t = new Tower(curtyp);
      t.setLevel(floor(random(0,height)));
      t.moveTo(x);
      twr[towers++] = t;
    }
  } 
  if(keyCode == ENTER) { 
    selectionSort();
    // assign locations evenly spread from 0 to stage width
    for (int i=0; i<towers; i++) {
      float d = width/(towers+1);
      float ll = (i+0.5) * d;
      twr[i].moveTo(floor(ll));
    }
  }
}
